from enum import Enum


class SplitType(Enum):
    SEASON = "Season"
    ARENA = "Place"
    ALL_STAR = "All_Star"
    RESULT = "Result"
    MONTH = "Month"
    CONFERENCE = "Conference"
    DIVISION = "Division"
    OPPONENT = "Opponent"
